/*
 ****************************************************************
 *
 * Copyright (C) 1998 Atsushi Umeshima <ume@tka.att.ne.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>

#include <X11/xpm.h>
#include "wmgeneral/wmgeneral.h"
#include "wmamfm-master.xpm"
#include "wmamfm-mask.xbm"
#include "amfm.h"

// Defines
#define NAME "wmamfm"
#define CLASS "WMAmfm"
#define BACKCOLOR "#282828"
#define VERSION "1.0.6"
#define RCFILE "wmamfmrc"
#define FCODE(f)    ((unsigned int)(((200.0*(double)(f)-15200.0)/10.0)+0.5))
#define ACODE(f)    ((unsigned int)(((double)(f)-531.0)/9.0))
#define FMFREQ_MIN 76.0
#define FMFREQ_MAX 108.0
#define AMFREQ_MIN 531.0
#define AMFREQ_MAX 1602.0
#define VOL_MIN 0
#define VOL_MAX 30
#define VOLUME_MIN 0
#define VOLUME_MAX 100
#define INT_SHORT 10000
#define INT_LONG  50000

// Global Variables
char *myname;
char temp[128];
XEvent Event;
int but_stat=-1;
int but_detail=-1;
int but_timer=0;
struct {
    double freq;
} *fm_presets, *am_presets;
char bckcol[256] = BACKCOLOR;
unsigned int fm_max_presets=0;
unsigned int am_max_presets=0;
unsigned int fm_preset_count=0;
unsigned int am_preset_count=0;
unsigned int preset_count=0;
int radio_status=OFF;
int radio_band=FM;
double volume=0.0;
double am_freqf=AMFREQ_MIN;
double fm_freqf=FMFREQ_MIN;
double freqf=FMFREQ_MIN;
char freqstr[20]="";
char *device=DEVICE;
int mixerfd;
int fd;
int interval = INT_LONG;
char *short_opts = "hvn";
struct option long_opts[] =
{
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'v'},
    {"on", 0, 0, 'n'},
    {NULL, 0, 0, 0}
};

// Functions
void dummy_handler(void);
void exit_radio(void);
void wmamfm_routine(int argc, char *argv[]);
void parse_command_line(int argc, char *argv[]);
int parse_rc_file(char *filename);
double check_freqf(double freq, int band);
void check_volume(void);
void usage(char *name);
void button_down(int button);
void button_up(int button);
void tune_up(void);
void tune_down(void);
void toggle_power(void);
void toggle_band(void);
void preset_update(void);
void volume_set(int vol);
void general_freq_update(int sec);
void preset_handler(int preset_hint);
void draw_digital_preset(void);
void draw_digital_freq(void);
void draw_volume_bar(int vol);
void draw_digital_band(void);
void draw_power_led(void);
void version(void);
void make_freqstr(void);
void radio_on(void);
void radio_off(void);
void radio_tune(int sec);

// Main
void main(int argc, char *argv[])
{
    myname = argv[0];
    parse_command_line(argc,argv);
    strcpy(temp,(char*)getenv("HOME"));
    strcat(temp,"/.");
    strcat(temp,RCFILE);
    if (parse_rc_file(temp) == 1)
    {
        strcpy(temp, "/usr/local/share/wmamfm/system.");
        strcat(temp, RCFILE);
        if (parse_rc_file(temp) == 1)
        {
            fprintf(stderr,"User(~/.%s) or global(/usr/local/share/wmamfm/system.%s) configuration files not found.\n",RCFILE,RCFILE);
            exit(1);
        }
    }

    mixerfd = openmixer();
    addrecsrc(mixerfd, SOUND_MIXER_LINE);
    signal(SIGALRM, (void *)dummy_handler);
    alarm(1);
    fd = radioinit(device);
    alarm(0);
    
    signal(SIGALRM, (void *)dummy_handler);
    alarm(1);
    if(radioprobe(fd))
    {
        radioclose(fd);
        fd = radioinit(device);
        signal(SIGALRM, (void *)dummy_handler);
        alarm(1);
        if(radioprobe(fd))
        {
            fprintf(stderr,"Radio not found.\n");
            radioclose(fd);
            closemixer(mixerfd);
            exit(1);
        }
    }
    alarm(0);
    signal(SIGALRM, SIG_DFL);
    signal(SIGINT, (void *)exit_radio);
    signal(SIGTERM, (void *)exit_radio);
    if(radio_status == ON)
    {
        radio_on();
    }
    else
    {
        radio_off();
    }
    wmamfm_routine(argc, argv);
    exit(0);
}

void dummy_handler(void) {
    signal(SIGALRM, SIG_IGN);
    return;
}

void exit_radio(void) {
    radio_off();
    radioclose(fd);
    closemixer(mixerfd);
    exit(0);
}

void radio_on(void)
{
        setvolume(mixerfd, 0);
        radioon(fd);
        if(freqf>0.0)
        {
            radiotune(fd, freqstr, radio_band);
        }
        usleep(75000);
        setvolume(mixerfd, volume);
}

void radio_off(void)
{
        setvolume(mixerfd, 0);
        radiooff(fd);
        usleep(50000);
        setvolume(mixerfd, volume);
}

void radio_tune(int sec)
{
        setvolume(mixerfd, 0);
        radiotune(fd, freqstr, radio_band);
        usleep(sec);
        setvolume(mixerfd, volume);
}

void wmamfm_routine(int argc, char *argv[])
{
    int i=-1;
 
    openXwindow(argc,argv,wmamfm_master_xpm,wmamfm_mask_bits,
                wmamfm_mask_width,wmamfm_mask_height);
    AddMouseRegion(0,4,4,12,13);		// FINE TUNE UP
    AddMouseRegion(1,4,15,12,24);		// FINE TUNE DOWN
    AddMouseRegion(2,4,27,12,41);		// LEFT PRESET SCAN BUTTON
    AddMouseRegion(3,39,27,47,41);		// RIGHT PRESET SCAN BUTTON
    AddMouseRegion(4,4,44,24,59);		// ON/OFF BUTTON
    AddMouseRegion(5,25,44,45,59);		// FM/AM BUTTON
    AddMouseRegion(6,50,29,58,59);		// VOLUME BAR
    AddMouseRegion(7,13,4,59,24);		// VOLUME BAR
    AddMouseRegion(8,15,27,36,41);		// VOLUME BAR

    volume_set((int)(volume*3/10));
    draw_power_led();
    general_freq_update(0);
    
    while (1)
    {
        
	if (but_stat < 0)
        {
            XPeekEvent(display, &Event);
        }
        
	while (XPending(display))
	{
            XNextEvent(display, &Event);
            switch (Event.type) 
            {
                case MapNotify:
                    RedrawWindow();
                    break;
                case Expose:
                    RedrawWindow();
                    break;
                case DestroyNotify:
                    XCloseDisplay(display);
                    exit_radio();
                    break;
                case ButtonPress:
                    i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
                    switch (i)
                    {
                        case 0:			// FINE TUNE UP
                            button_down(0);
                            but_timer = 0;
                            break;
                        case 1:			// FINE TUNE DOWN
                            button_down(1);
                            but_timer = 0;
                            break;
                        case 2:			// LEFT PRESET SCAN BUTTON
                            button_down(2);
                            break;
                        case 3:			// RIGHT PRESET SCAN BUTTON
                            button_down(3);
                            break;
                        case 4:			// ON/OFF BUTTON
                            button_down(4);
                            break;
                        case 5:			// FM/AM BUTTON
                            button_down(5);
                            break;
                        case 6:			// VOLUME BAR
                            button_down(6);
                            volume_set(59-Event.xbutton.y);
                        case 7:			// FINE TUNE UP
                            but_timer = 0;
                            break;
                    }
                    but_stat = i;
                    but_detail = Event.xbutton.button;
                    break;
                case ButtonRelease:
                    interval = INT_LONG;
                    switch (but_stat) 
                    {
                        case 0:			// FINE TUNE UP
                            button_up(0);
                            break;
                        case 1:			// FINE TUNE DOWN
                            button_up(1);
                            break;
                        case 2:			// LEFT PRESET SCAN BUTTON
                            button_up(2);
                            break;
                        case 3:			// RIGHT PRESET SCAN BUTTON
                            button_up(3);
                            break;
                        case 4:			// ON/OFF BUTTON
                            button_up(4);
                            break;
                        case 5:			// FM/AM BUTTON
                            button_up(5);
                            break;
                        case 6:			// VOLUME BAR
                            button_up(6);
                            break;
                    }
                    if (i == but_stat && but_stat >= 0)
                    {
                        switch (i)
                        {
                            case 0:			// FINE TUNE UP
                                if (!(but_timer >= 10))
                                {
                                    tune_up();
                                }
                                break;
                            case 1:			// FINE TUNE DOWN
                                if (!(but_timer >= 10))
                                {
                                    tune_down();
                                }
                                break;
                            case 2:			// LEFT PRESET SCAN BUTTON
                                preset_handler(-1);
                                break;
                            case 3:			// RIGHT PRESET SCAN BUTTON
                                preset_handler(1);
                                break;
                            case 4:			// ON/OFF BUTTON
                                toggle_power();
                                break;
                            case 5:			// FM/AM BUTTON
                                toggle_band();
                                break;
                            case 6:			// VOLUME BAR
                                volume_set(59-Event.xbutton.y);
                                break;
                            case 7:			// DIGITAL FREQ BOX
                                switch (Event.xbutton.button)
                                {
                                    case Button3:		
                                    case Button2:		
                                        if (!(but_timer >= 10))
                                        {
                                            tune_up();
                                        }
                                        break;
                                    case Button1:		
                                        if (!(but_timer >= 10))
                                        {
                                            tune_down();
                                        }
                                        break;
                                }
                                break;
                            case 8:			// DIGITAL PRESETS BOX
                                switch (Event.xbutton.button)
                                {
                                    case Button3:		
                                    case Button2:		
                                        preset_handler(1);
                                        break;
                                    case Button1:		
                                        preset_handler(-1);
                                        break;
                                }
                                break;
                        }
                    }
                    but_stat = -1;
                    but_detail = -1;
                    i = -1;
            }
	}
        
	if (i == but_stat && but_stat >= 0)
	{
            but_timer++;
            if(but_timer == 50)
            {
                interval = INT_SHORT;
            }
            
            switch (i)
            {
                case 6:			// VOLUME BAR
                    volume_set(59-Event.xbutton.y);
                    break;
            }
            
            if(but_timer >= 10)
            {
                switch (i)
                {
                    case 0:			// FINE TUNE UP (+) AUTO-REPEAT
                        tune_up();
                        break;
                    case 1:			// FINE TUNE DOWN (-) AUTO-REPEAT
                        tune_down();
                        break;
                    case 7:			// FINE TUNE UP (+) AUTO-REPEAT
                        switch(but_detail)
                        {
                            case Button3:		
                            case Button2:		
                                tune_up();
                                break;
                            case Button1:
                                tune_down();
                                break;
                        }
                        break;
                }
            }
	}
        volume = getvolume(mixerfd);
        volume_set((int)(volume*3/10));
	usleep(interval);
    }
}

void parse_command_line(int argc, char *argv[])
{
    int opt_index = 0, c;
    
    while ((c = getopt_long(argc, argv, short_opts, long_opts, &opt_index)) != EOF) {
        switch( c ) {
            case 'h':
                usage( argv[0] );
                exit(0);
                break;
            case 'v':
                version();
                exit(0);
                break;
            case 'n':
                radio_status = ON;
                break;
            default:
                exit(1);
        }
    }
}

int parse_rc_file(char *filename)
{
    char temp[128];
    char *p;
    FILE *fp;
    int fm_count=1;
    int am_count=1;
    char *tokens = " \t\n";
    
    if ((fp = fopen(filename,"r")) == NULL)
    {
        return 1;
    }
    
    while(fgets(temp, 128, fp)) 
    {
        if ((p = strtok(temp,tokens)) != NULL)
        {
            if ((p = strchr(temp,'#')) != NULL) 
            {
                *p = '\0';
            }
            if ((strlen(temp)) != 0)
            {
                if (strcmp(temp, "FMStation:") == 0)
                {
                    fm_max_presets++;
                }
                else if (strcmp(temp, "AMStation:") == 0)
                {
                    am_max_presets++;
                }
            }
        }
    }

    fm_presets = malloc(sizeof(*fm_presets) * (fm_max_presets+1));
    am_presets = malloc(sizeof(*am_presets) * (am_max_presets+1));
    fm_presets[0].freq = FMFREQ_MIN;
    am_presets[0].freq = AMFREQ_MIN;
    
    fseek(fp, 0L, SEEK_SET);
    
    while(fgets(temp, 128, fp)) 
    {
        if ((p = strtok(temp,tokens)) != NULL)
        {
            if ((p = strchr(temp,'#')) != NULL) 
            {
                *p = '\0';
            }
            if ((strlen(temp)) != 0)
            {
                if ((p = strtok(NULL,tokens)) != NULL)
                {
                    if (strcmp(temp, "Device:") == 0)
                    {
                        device = (char *)malloc(strlen(p)+1);
                        strcpy(device, p);
                    }
                    else if (strcmp(temp, "FMStation:") == 0)
                    {
                        sscanf(p,"%lf",&fm_presets[fm_count].freq);
                        fm_presets[fm_count].freq =
                            check_freqf(fm_presets[fm_count].freq, FM);
                        fm_count++;
                    }
                    else if (strcmp(temp, "AMStation:") == 0)
                    {
                        sscanf(p,"%lf",&am_presets[am_count].freq);
                        am_presets[am_count].freq =
                            check_freqf(am_presets[am_count].freq, AM);
                        am_count++;
                    }
                    else if (strcmp(temp, "Startup:") == 0)
                    {
                        if (strcmp(p, "AM") == 0)
                        {
                            radio_band = AM;
                        }
                        if (strcmp(p, "FM") == 0)
                        {
                            radio_band = FM;
                        }
                        if ((p = strtok(NULL,tokens)) != NULL)
                        {
                            sscanf(p,"%lf",&freqf);
                            if (radio_band == FM)
                            {
                                freqf = check_freqf(freqf, FM);
                                fm_freqf = freqf;
                            }
                            else
                            {
                                freqf = check_freqf(freqf, AM);
                                am_freqf = freqf;
                            }
                        }
                        if ((p = strtok(NULL,tokens)) != NULL)
                        {
                            sscanf(p,"%lf",&volume);
                            check_volume();
                        }
                    }
                }
            }
        }
    }

    make_freqstr();
    
    fclose(fp);
    return 0;
}

void make_freqstr(void)
{
    if(radio_band == FM)
    {
        sprintf(freqstr, "%03d", FCODE(freqf));
    }
    else
    {
        sprintf(freqstr, "%03d", ACODE(freqf));
    }
}

double check_freqf(double freq, int band)
{
    if (band == FM)
    {
        if(freq > FMFREQ_MAX)
        {
            freq = FMFREQ_MAX;
        }
        else if(freq < FMFREQ_MIN)
        {
            freq = FMFREQ_MIN;
        }
    }
    else
    {
        if(freq > AMFREQ_MAX)
        {
            freq = AMFREQ_MAX;
        }
        else if(freq < AMFREQ_MIN)
        {
            freq = AMFREQ_MIN;
        }
    }
    return freq;
}

void check_volume(void)
{
    if(volume > VOLUME_MAX)
    {
        volume = VOLUME_MAX;
    }
    else if(volume < VOLUME_MIN)
    {
        volume = VOLUME_MIN;
    }
}

void usage(char *name)
{
  version();

  printf("\n");
  printf("Usage: %s [OPTIONS]\n", myname);
  printf("\n");
  printf("  -n, --on                       "
         "turn radio on initially on startup\n");
  printf("  -h, --help                     "
         "display this help and exit\n");
  printf("  -v, --version                  "
         "display the version and exit\n");
  printf("\n");
  printf("Author: Atsushi Umeshima <ume@tka.att.ne.jp>\n");
  printf("   Web: http://members.xoom.com/ume/amfm.html\n");
}

void button_down(int button)
{
    switch (button)
    {
        case 0:			// FINE TUNE UP
            copyXPMArea(50, 103, 7, 10, 4, 4);
            RedrawWindowXYWH(4, 4, 7, 10);
            break;
        case 1:			// FINE TUNE DOWN
            copyXPMArea(65, 103, 7, 10, 4, 15);
            RedrawWindowXYWH(4, 15, 7, 10);
            break;
        case 2:			// LEFT PRESET SCAN BUTTON
            copyXPMArea(52, 86, 9, 15, 4, 27);
            RedrawWindowXYWH(4, 27, 9, 15);
            break;
        case 3:			// RIGHT PRESET SCAN BUTTON
            copyXPMArea(71, 86, 9, 15, 39, 27);
            RedrawWindowXYWH(39, 27, 9, 15);
            break;
        case 4:			// ON/OFF BUTTON
            copyXPMArea(21, 103, 21, 16, 4, 44);
            RedrawWindowXYWH(4, 44, 21, 16);
            break;
        case 5:			// FM/AM BUTTON
            copyXPMArea(21, 86, 21, 16, 25, 44);
            RedrawWindowXYWH(25, 44, 21, 16);
            break;
        case 6:			// VOLUME BAR
            break;
    }
}

void button_up(int button)
{
    switch (button)
    {
        case 0:			// FINE TUNE UP
            copyXPMArea(43, 103, 7, 10, 4, 4);
            RedrawWindowXYWH(4, 4, 7, 10);
            break;
        case 1:			// FINE TUNE DOWN
            copyXPMArea(58, 103, 7, 10, 4, 15);
            RedrawWindowXYWH(4, 15, 7, 10);
            break;
        case 2:			// LEFT PRESET SCAN BUTTON
            copyXPMArea(43, 86, 9, 15, 4, 27);
            RedrawWindowXYWH(4, 27, 9, 15);
            break;
        case 3:			// RIGHT PRESET SCAN BUTTON
            copyXPMArea(62, 86, 9, 15, 39, 27);
            RedrawWindowXYWH(39, 27, 9, 15);
            break;
        case 4:			// ON/OFF BUTTON
            copyXPMArea(0, 103, 21, 16, 4, 44);
            RedrawWindowXYWH(4, 44, 21, 16);
            break;
        case 5:			// FM/AM BUTTON
            copyXPMArea(0, 86, 21, 16, 25, 44);
            RedrawWindowXYWH(25, 44, 21, 16);
            break;
        case 6:			// VOLUME BAR
            break;
    }
}

void tune_up(void)
{
    if(radio_band == FM)
    {
	if ((float)freqf != FMFREQ_MAX)
	{
            freqf += .05;
	}
	else
	{
            freqf = FMFREQ_MIN;
	}
        fm_freqf = freqf;
    }
    else
    {
	if ((float)freqf != AMFREQ_MAX)
	{
            freqf += 9.0;
	}
	else
	{
            freqf = AMFREQ_MIN;
	}
        am_freqf = freqf;
    }
    general_freq_update(0);
}

void tune_down(void)
{
    if(radio_band == FM)
    {
	if ((float)freqf != FMFREQ_MIN)
	{
            freqf -= .05;
	}
	else
	{
            freqf = FMFREQ_MAX;
	}
        fm_freqf = freqf;
    }
    else
    {
	if ((float)freqf != AMFREQ_MIN)
	{
            freqf -= 9.0;
	}
	else
	{
            freqf = AMFREQ_MAX;
	}
        am_freqf = freqf;
    }
    general_freq_update(0);
}

void toggle_power(void)
{
    if(radio_status == OFF)
    {
        radio_status = ON;
        radio_on();
    }
    else
    {
        radio_status = OFF;
        radio_off();
    }
    draw_power_led();
}

void toggle_band(void)
{
    if(radio_band == FM)
    {
        radio_band = AM;
        freqf = am_freqf;
    }
    else
    {
        radio_band = FM;
        freqf = fm_freqf;
    }
    general_freq_update(0);
}

void preset_update(void)
{
    int i;
    preset_count = 0;
    if(radio_band == FM)
    {
        for (i=1;i<=fm_max_presets;i++)
        {
            if (freqf == fm_presets[i].freq)
            {
                preset_count = i;
                fm_preset_count = preset_count;
            }
        }
    }
    else
    {
        for (i=1;i<=am_max_presets;i++)
        {
            if (freqf == am_presets[i].freq)
            {
                preset_count = i;
                am_preset_count = preset_count;
            }
        }
    }
}

void volume_set(int vol)
{
    if(vol > VOL_MAX)
    {
        vol = VOL_MAX;
    }
    else if(vol < VOL_MIN)
    {
        vol = VOL_MIN;
    }
    volume = vol*10/3;
    setvolume(mixerfd, (int)volume);
    draw_volume_bar(vol);
}


void general_freq_update(int sec)
{
    make_freqstr();
    if(radio_status == ON)
    {
        radio_tune(50000);
    }
    preset_update();
    draw_digital_preset();
    draw_digital_freq();
    draw_digital_band();
}

void preset_handler(int preset_hint)
{
    if(radio_band == FM)
    {
        preset_count = fm_preset_count;
    }
    else
    {
        preset_count = am_preset_count;
    }
    
    if (preset_hint < 0)
    {
        if (preset_count <= 1)
        {
            if(radio_band == FM)
            {
                preset_count = fm_max_presets;
            }
            else
            {
                preset_count = am_max_presets;
            }
        }
        else
        {
            preset_count--;
        }
    } 
    else
    {
        if(radio_band == FM)
        {
            if(fm_max_presets > 0)
            {
                if(preset_count == fm_max_presets)
                {
                    preset_count = 1;
                }
                else
                {
                    preset_count++;
                }
            }
        }
        else
        {
            if(am_max_presets > 0)
            {
                if(preset_count == am_max_presets)
                {
                    preset_count = 1;
                }
                else
                {
                    preset_count++;
                }
            }
        }
    }
        
    if(radio_band == FM)
    {
        fm_preset_count = preset_count;
        freqf = fm_presets[fm_preset_count].freq;
        fm_freqf = freqf;
    }
    else
    {
        am_preset_count = preset_count;
        freqf = am_presets[am_preset_count].freq;
        am_freqf = freqf;
    }

    general_freq_update(50000);
}

void draw_digital_preset(void)
{
    char temp[10];
    char *p = temp;
    int j=0,k=17;

    sprintf(temp, "%02d", preset_count);

    while (*p == '0' && j<2)
    {
        copyXPMArea(93, 64, 9, 11, k, 29);
        k += 9;
        j++;
        p++;
    }
    while (j<2) 
    {
        copyXPMArea((*p-'0')*9, 64, 9, 11, k, 29);
        k += 9;
        p++;
        j++;
    }
    RedrawWindowXYWH(17, 29, 18, 11);
}
	
void draw_digital_freq(void)
{
    char temp[10];
    char *p = temp;
    int j=0,k=15;

    if(radio_band == FM)
    {
        sprintf(temp,"%06.2f",freqf);
            
        if (*p == '1')
        {
            copyXPMArea(13, 64, 4, 11, k, 6);
            k += 4;
            j++;
            p++;
        }
        else
        {
            copyXPMArea(93, 64, 4, 11, k, 6);
            k += 4;
            j++;
            p++;
        }
        while (j<6) 
        {
            if (*p == '.')
            {
                copyXPMArea(90, 64, 3, 11, k, 6);
                k += 3;
            }
            else
            {
                copyXPMArea((*p-'0')*9, 64, 9, 11, k, 6);
                k += 9;
            }
            p++;
            j++;
        }
    }
    else
    {
        sprintf(temp,"%04.0f",freqf);
            
        copyXPMArea(93, 64, 4, 11, k, 6);
        k += 4;
            
        if (*p == '0')
        {
            copyXPMArea(93, 64, 9, 11, k, 6);
            k += 9;
            p++;
            j++;
        }
        while (j<4) 
        {
            copyXPMArea((*p-'0')*9, 64, 9, 11, k, 6);
            k += 9;
            p++;
            j++;
        }
        copyXPMArea(93, 64, 3, 11, k, 6);
    }
    RedrawWindowXYWH(15, 6, 43, 11);
}

void draw_volume_bar(int vol)
{
    if(vol < VOL_MAX)
    {
        copyXPMArea(90, 86, 9, VOL_MAX-vol, 50, 29);
    }
    if(vol > VOL_MIN)
    {
        copyXPMArea(81, 86+VOL_MAX-vol, 9, vol, 50, 29+VOL_MAX-vol); 
    }
    RedrawWindowXYWH(50, 29, 9, 30); 
}
    
void draw_digital_band(void)
{
    if(radio_band == FM)
    {
	copyXPMArea(97, 77, 12, 6, 29, 49); // FM
	copyXPMArea(103, 64, 13, 5, 45, 18); // MHz
    }
    else
    {
	copyXPMArea(110, 77, 12, 6, 29, 49); // AM
	copyXPMArea(103, 70, 13, 5, 45, 18); // KHz
    }
    RedrawWindowXYWH(29, 49, 12, 6); // FM/AM
    RedrawWindowXYWH(45, 18, 13, 5); // MHz/KHz
}

void draw_power_led(void)
{
    if(radio_status == ON)
    {
	copyXPMArea(58, 114, 15, 3, 7, 47); 
    }
    else
    {
	copyXPMArea(43, 114, 15, 3, 7, 47); 
    }
    RedrawWindowXYWH(7, 47, 15, 3);
}

void version(void)
{
  printf("This is " NAME " " VERSION "\n");
  printf("Copyright (C) 1998 Atsushi Umeshima\n");
}
